#include <Adafruit_GFX_AS.h>    // Core graphics library, with extra fonts.
#include <Adafruit_ILI9341_STM.h> // STM32 DMA Hardware-specific library
#define TFT_DC PB1
#define TFT_CS PB10
#define TFT_RST PB11
Adafruit_ILI9341_STM tft = Adafruit_ILI9341_STM(TFT_CS, TFT_DC, TFT_RST); // Use hardware SPI

#include <SPI.h>
#include <nRF24L01.h>
#include <RF24.h>
//#include <printf.h>

#include <EEPROM.h>

const uint64_t pipeOut = 0xE8E8F0F0E1LL;

byte oc0 = 173;
byte oc1 = 173;
byte oc2 = 173;
byte oc3 = 159;

byte menu = 0;
bool menuChange = 1;
byte radioPower = 1;

RF24 radio(PA15,PB12);

// The sizeof this struct should not exceed 32 bytes
struct MyData {
  byte c0;
  byte c1;
  byte c2;
  byte c3;
};

MyData data;

void resetData() 
{
  data.c0 = 0;
  data.c1 = 127;
  data.c2 = 127;
  data.c3 = 127;
}

void setRadio(byte power, byte dataRate){

  if(power==0){
    radio.setPALevel(RF24_PA_MIN);
    }
  else if(power==1){
    radio.setPALevel(RF24_PA_LOW);
    }
  else if(power==2){
    radio.setPALevel(RF24_PA_HIGH);
    }
  else if(power==3){
    radio.setPALevel(RF24_PA_MAX);
    }

  if(dataRate==0){
    radio.setDataRate(RF24_250KBPS);
    }
  else if(dataRate==1){
    radio.setDataRate(RF24_250KBPS);
    }
  else if(dataRate==2){
    radio.setDataRate(RF24_250KBPS);
    }
  else if(dataRate==3){
    radio.setDataRate(RF24_250KBPS);
    }
  
  }
void setup()
{
  //Serial.begin(9600);
  //Serial.print("ON");
  //printf_begin();
  
  //TFT
  tft.begin();
  tft.fillScreen(ILI9341_BLACK);
  tft.setTextColor(ILI9341_WHITE,ILI9341_BLACK);  
  tft.setTextSize(2);
  
  //Buttons
  pinMode(PB3, INPUT_PULLUP);//Mode
  pinMode(PB4, INPUT_PULLUP);//Ch
  pinMode(PB5, INPUT_PULLUP);//Dec
  pinMode(PB8, INPUT_PULLUP);//Inc
  //Sticks
  pinMode(PA0, INPUT_ANALOG);
  pinMode(PA1, INPUT_ANALOG);
  pinMode(PA2, INPUT_ANALOG);
  pinMode(PA3, INPUT_ANALOG);
  
  radio.begin();
  radio.setPALevel(RF24_PA_LOW);
  radio.setAutoAck(false);
  radio.setDataRate(RF24_250KBPS);

  //radio.printDetails();

  radio.openWritingPipe(pipeOut);
  resetData();

  tft.fillScreen(ILI9341_BLACK);
  tft.drawFastHLine(0,160,240,ILI9341_WHITE);
  tft.drawFastHLine(0,161,240,ILI9341_WHITE);
  
}

void loop()
{
  gui();
  
  data.c0 = (analogRead(PA0)/12)-oc0 +127;
  if(data.c0>255){data.c0=255;}
  if(data.c0<0){data.c0=0;}
  data.c1 = analogRead(PA1)/12-oc1 +127;
  if(data.c1>255){data.c1=255;}
  if(data.c1<0){data.c1=0;}
  data.c2 = analogRead(PA2)/12-oc2 +127;
  if(data.c2>255){data.c2=255;}
  if(data.c2<0){data.c2=0;}
  data.c3 = analogRead(PA3)/12-oc3 +127;
  if(data.c3>255){data.c3=255;}
  if(data.c3<0){data.c3=0;}

  radio.write(&data, sizeof(MyData));
  
}


void gui(){
    tft.drawPixel(data.c1,data.c0,ILI9341_GREEN);
    tft.drawPixel(data.c3,data.c2,ILI9341_RED);
    
  if(digitalRead(PB3)==0){
    delay(100);
    if(menu<2){menu++;}
    else{menu=0;}
    tft.fillRect(0,162,240,150,ILI9341_BLACK);
    menuChange=1;
    }
  
  if(menu==0){
      if(menuChange==1){
        tft.setCursor(0, 170);
        tft.print("Data Sent");
        tft.setCursor(0, 210);
        tft.print("Raw Inputs");
        tft.setCursor(0, 250);
        tft.print("Stick Offsets");
      
        tft.setCursor(0, 270);
        tft.print(oc0);
        tft.print(" ");
        tft.setCursor(60, 270);
        tft.print(oc2);
        tft.print(" ");
        tft.setCursor(120, 270);
        tft.print(oc2);
        tft.print(" ");
        tft.setCursor(180, 270);
        tft.print(oc3);
        tft.print(" ");
      }
    menuChange=0;
    
    tft.setCursor(0, 190);
    tft.print(data.c0);
    tft.print(" ");
    tft.setCursor(60, 190);
    tft.print(data.c1);
    tft.print(" ");
    tft.setCursor(120, 190);
    tft.print(data.c2);
    tft.print(" ");
    tft.setCursor(180, 190);
    tft.print(data.c3);
    tft.print(" ");
  
    tft.setCursor(0, 230);
    tft.print(analogRead(PA0));
    tft.print(" ");
    tft.setCursor(60, 230);
    tft.print(analogRead(PA1));
    tft.print(" ");
    tft.setCursor(120, 230);
    tft.print(analogRead(PA2));
    tft.print(" ");
    tft.setCursor(180, 230);
    tft.print(analogRead(PA3));
    tft.print(" ");
  }

  else if(menu==1){
    if(menuChange==1){
      tft.setCursor(15, 170);
      tft.print("Please center the");
      tft.setCursor(15, 190);
      tft.print("sticks then press");
      tft.setCursor(90, 210);
      tft.print("ENTER");
      tft.setCursor(0, 250);
      tft.print("Stick Offsets");
  
      tft.setCursor(0, 270);
      tft.print(oc0);
      tft.print("  ");
      tft.setCursor(60, 270);
      tft.print(oc2);
      tft.print("  ");
      tft.setCursor(120, 270);
      tft.print(oc2);
      tft.print("  ");
      tft.setCursor(180, 270);
      tft.print(oc3);
      tft.print("  ");
      }
    menuChange=0;
    if(digitalRead(PB4)==0 && digitalRead(PB5)==0){
      delay(100);
      oc0 = analogRead(PA0)/12;
      oc1 = analogRead(PA1)/12;
      oc2 = analogRead(PA2)/12;
      oc3 = analogRead(PA3)/12;
      tft.setTextColor(ILI9341_BLACK,ILI9341_WHITE);
      tft.setCursor(0, 270);
      tft.print(oc0);
      tft.print("  ");
      tft.setCursor(60, 270);
      tft.print(oc2);
      tft.print("  ");
      tft.setCursor(120, 270);
      tft.print(oc2);
      tft.print("  ");
      tft.setCursor(180, 270);
      tft.print(oc3);
      tft.print("  ");
      tft.setTextColor(ILI9341_WHITE,ILI9341_BLACK);
      menuChange=1;
      }
    }

      else if(menu==2){
    if(menuChange==1){
      tft.setCursor(0, 170);
      tft.print("Radio Settings");
      tft.setCursor(0, 190);
      tft.print("Power:");
      tft.setCursor(120, 190);
      if(radioPower==0){
        tft.print("Min ");
        }
      else if(radioPower==1){
        tft.print("Low ");
        }
      else if(radioPower==2){
        tft.print("High ");
        }
      else if(radioPower==3){
        tft.print("Max ");
        }
      tft.setCursor(0, 210);
      tft.print("Data Rate:");

      }
    menuChange=0;
    if(digitalRead(PB8)==0){
      delay(100);
    if(radioPower<4){
      radioPower++;
      }
    else{
      radioPower=0;
    }
      setRadio(radioPower, 0);

      menuChange=1;
      }
    if(digitalRead(PB5)==0){
      delay(100);
      
    if(radioPower>1){
      radioPower--;
      }
    else{
      radioPower=4;
    }
      setRadio(radioPower, 0);

      menuChange=1;
      }
    }



}



